@@ -1,5 +1,6 @@
 /*
-Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008,2009
+Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008,2009,
+              2010,2011,2012
 Whitehead Institute for Biomedical Research, Steve Rozen
 (http://purl.com/STEVEROZEN/), Andreas Untergasser and Helen Skaletsky
 All rights reserved.
@@ -60,6 +61,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define PR_DEFAULT_PRODUCT_MAX_TM     1000000.0
 #define PR_DEFAULT_PRODUCT_MIN_TM     -1000000.0
 
+#define PR_NULL_FORCE_POSITION       -1000000
+
 /*  Exposed in the read_boulder input routine.... */
 #define PR_NULL_START_CODON_POS       -1000000
 #define PR_DEFAULT_START_CODON_POS    PR_NULL_START_CODON_POS
@@ -81,16 +84,19 @@ if (!(COND)) {                                           \
     abort();                                             \
 }
 
-/* ANDREAS, we need documentation on these */
 /* Enum to define tasks primer3 can do */
 typedef enum task { 
   pick_pcr_primers               = 0,
-  /*  For backward compatibility, equivalent to pick_detection_primers */
+  /*  For backward compatibility, equivalent to
+      generic
+      plus pick_left_primer =1
+      plus pick_right_primer = 1
+      plus pick_internal_oligo = 0 */
   pick_pcr_primers_and_hyb_probe = 1,
   pick_left_only                 = 2,
   pick_right_only                = 3,
   pick_hyb_probe_only            = 4,
-  pick_detection_primers         = 5,
+  generic                        = 5,
   pick_cloning_primers           = 6,
   pick_discriminative_primers    = 7,    
   pick_sequencing_primers        = 8,
@@ -116,26 +122,28 @@ typedef struct pr_append_str {
  * "Global" arguments in the documentation.)
  */
 typedef struct oligo_weights {
-  double temp_gt;
-  double temp_lt;
-  double gc_content_gt;
-  double gc_content_lt;
+
   double compl_any;
   double compl_any_th;
   double compl_end;
   double compl_end_th;
-  double hairpin_th;
-  double temp_cutoff;
-  double repeat_sim;
-  double length_lt;
-  double length_gt;
-  double seq_quality;
   double end_quality;
-  double pos_penalty;
   double end_stability;
+  double gc_content_gt;
+  double gc_content_lt;
+  double hairpin_th;
+  double length_gt;
+  double length_lt;
   double num_ns;
+  double pos_penalty;
+  double repeat_sim;
+  double seq_quality;
+  double temp_cutoff;
+  double temp_gt;
+  double temp_lt;
   double template_mispriming;
   double template_mispriming_th;
+
 } oligo_weights;
 
 typedef struct pair_weights {
@@ -416,7 +424,6 @@ typedef struct p3_global_settings {
   double pair_compl_any_th;
   double pair_compl_end;
   double pair_compl_end_th;
-  double pair_hairpin_th;
    
   int thermodynamic_alignment;
   /* 
@@ -459,20 +466,29 @@ typedef struct p3_global_settings {
 typedef enum oligo_type { OT_LEFT = 0, OT_RIGHT = 1, OT_INTL = 2 }
   oligo_type;
 
+/* This struct captures informatin about the similarity of
+   an oligo (primer) to elements in a mispriming (repeat)
+   library (which is read in from a fasta file). */
 typedef struct rep_sim {
-  char *name;       /* Name of the sequence from given file in fasta
-                     * format with maximum similarity to the oligo.
+  char *name;       /* Name of the sequence format with maximum
+		       similarity to the oligo.
                      */
+
   short min;        /* 
                      * The minimum score in slot 'score' (below).
                      * (Used when the objective function involves
                      * minimization of mispriming possibilities.)
                      */
-  short max;        /* The maximum score in slot 'score' (below). */
+
+  short max;        /* The index of the maximum score in slot 'score'
+		       (below). */
+
   double *score;    /* 
                     * Array of similarity (i.e. false-priming) scores,
                     * one for each entry in the 'repeat_lib' slot
-                    * of the primargs struct. 
+                    * of the primargs struct.  In libprimer3.c,
+		    * score is set to NULL to indicate that
+		    * the rep_sim structure is uninitialized.
                     */
 } rep_sim;
 
@@ -487,9 +503,8 @@ typedef struct oligo_problems {
 typedef struct primer_rec {
         
   rep_sim repeat_sim;
-  /* Name of the sequence from given file in fasta
-   * format with maximum similarity to the oligo
-   * and corresponding alignment score.
+  /* Information on the best repeat library (mispriming library)
+   * match for this oligo (primer), plus additional scores.
    */
         
   double temp; /* The oligo melting temperature calculated for the
@@ -666,6 +681,7 @@ typedef struct pair_stats {
   int high_tm;             /* Product Tm too high.                          */
   int low_tm;              /* Product Tm too low.                           */
   int template_mispriming; /* Sum of template mispriming scores too high.   */
+
   /* Neither oligo in the pairs overlaps one of the "required sites".       */
   int does_not_overlap_a_required_point;
 
@@ -676,7 +692,12 @@ typedef struct pair_stats {
      PRIMER_PAIR_OK_REGION_LIST. */
   int not_in_any_ok_region;
 
+  /* Left primer to the right of right right primer. This can occur when
+     the primers are provided by the caller. */
+  int reversed;
+
   int ok;                  /* Number that were ok.                          */
+
 } pair_stats;
 
 typedef struct pair_array_t {
@@ -699,8 +720,8 @@ typedef struct seq_args {
                            * "included region").
                            */
 
-  interval_array_t2 tar2; /* The targets.  tar2->pairs[i][0] s the start
-                           * of the ith target, tar[i][1] its length.  */
+  interval_array_t2 tar2; /* The targets.  tar2->pairs[i][0] is the start
+                           * of the ith target, tar2->pairs[i][1] its length.  */
 
   interval_array_t2 excl2;/* The number of excluded regions. */
 
@@ -909,7 +930,12 @@ int p3_set_sa_upcased_sequence(seq_args *sargs, const char *upcased_sequencd);
    set slots, get slots */
 /* ============================================================ */
 
+/* Use this for -default_version=2 (up-to-date defaults) */
 p3_global_settings *p3_create_global_settings();
+
+/* Use this for -default_version=1 (old defaults) */
+p3_global_settings *p3_create_global_settings_default_version_1();
+
 void p3_destroy_global_settings(p3_global_settings *);
 
 void p3_set_gs_prmin (p3_global_settings * p , int val, int i);
@@ -985,7 +1011,6 @@ void p3_set_gs_primer_pair_max_mispriming(p3_global_settings * p , double val);
 void p3_set_gs_primer_pair_max_mispriming_th(p3_global_settings * p , double val);
 void p3_set_gs_primer_max_template_mispriming(p3_global_settings * p , double val);
 void p3_set_gs_primer_max_template_mispriming_th(p3_global_settings * p , double val);
-void p3_set_gs_primer_internal_oligo_max_template_mishyb(p3_global_settings * p , double val);
 void p3_set_gs_primer_lib_ambiguity_codes_consensus(p3_global_settings * p , int val);
 void p3_set_gs_primer_inside_penalty(p3_global_settings * p , double val);
 void p3_set_gs_primer_outside_penalty(p3_global_settings * p , double val);
@@ -1027,7 +1052,6 @@ void p3_set_gs_primer_io_wt_num_ns(p3_global_settings * p , double val);
 void p3_set_gs_primer_io_wt_rep_sim(p3_global_settings * p , double val);
 void p3_set_gs_primer_io_wt_seq_qual(p3_global_settings * p , double val);
 void p3_set_gs_primer_io_wt_end_qual(p3_global_settings * p , double val);
-void p3_set_gs_primer_io_wt_template_mishyb(p3_global_settings * p , double val);
 void p3_set_gs_primer_pair_wt_pr_penalty(p3_global_settings * p , double val);
 void p3_set_gs_primer_pair_wt_io_penalty(p3_global_settings * p , double val);
 void p3_set_gs_primer_pair_wt_diff_tm(p3_global_settings * p , double val);
@@ -1083,7 +1107,6 @@ void p3_set_gs_pair_compl_any(p3_global_settings * p , double  pair_compl_any);
 void p3_set_gs_pair_compl_end(p3_global_settings * p , double  pair_compl_end);
 void p3_set_gs_pair_compl_any_th(p3_global_settings * p , double  pair_compl_any_th);
 void p3_set_gs_pair_compl_end_th(p3_global_settings * p , double  pair_compl_end_th);
-void p3_set_gs_pair_hairpin_th(p3_global_settings * p , double  pair_hairpin_th);
 
 void p3_set_gs_min_left_three_prime_distance(p3_global_settings *p, int min_distance);
 void p3_set_gs_min_right_three_prime_distance(p3_global_settings *p, int min_distance);
@@ -1156,27 +1179,52 @@ p3_set_program_name(const char *name);
  */
 char* p3_read_line(FILE *file);
 
+/* 
+   Return 1 iff the argument 'oligo' has any problems -- i.e.
+   violations of design constraints.
+ */
 int        p3_ol_has_any_problem(const primer_rec *oligo);
+
+/*  
+    Return a string details the the problems in 'oligo', i.e. the
+    constraints that 'oligo' violates.  WARNING: Returns a pointer to
+    static storage, which is over-written on next call.
+ */
 const char *p3_get_ol_problem_string(const primer_rec *oligo);
 
+
+/* 
+   Creates up to three files, the names of which are based on the
+   argument 'file'.  One file is a table of forward primers, one a table
+   of reverse primers, and one a table of internal hybridization
+   oligos, depending on what the caller to choose_primers() requested
+   (in the 'pa' argument).  Returns 0 on success, 1 on error.  On
+   error, check errno for ENOMEM. Used to implement P3_FILE_FLAG=1.
+ */
 int    p3_print_oligo_lists(const p3retval*, 
                             const seq_args *, 
                             const p3_global_settings *, 
                             pr_append_str *err,
                             const char* file_name);
 
-/* Must be called from main to allow dependency on 
- * input version=3 in future version can it be 
- * deleted.                                       */
+
+/* 
+   Translate the values in the stats struct into an warning string.
+   Call this function only if the 'stat's contains the _errors_
+   associated with a given primer i.e. that primer was supplied by the
+   caller and pick_anyway is set.
+*/
 void add_must_use_warnings(pr_append_str *warning,
-                      const char* text,
-                      const oligo_stats *stats);
+			   const char* text,
+			   const oligo_stats *stats);
 
 
 
-/* Reverse and complement the sequence seq and put the result in s.
+/* 
+   Reverse and complement the sequence seq and put the result in s.
    WARNING: It is up the caller to ensure that s points to enough
-   space. */ 
+   space.
+*/ 
 void   p3_reverse_complement(const char *, char *);
 
 #ifdef __cplusplus
