@@ -1,5 +1,6 @@
 /*
-Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008
+Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008,2009,
+              2010,2011,2012
 Whitehead Institute for Biomedical Research, Steve Rozen
 (http://purl.com/STEVEROZEN/), Andreas Untergasser and Helen Skaletsky.
 All rights reserved.
@@ -111,7 +112,7 @@ extern double strtod();
                                "Duplicate tag: "); \
            pr_append(parse_err, TAG);              \
        } else {                                    \
-           T = (char*) _rb_safe_malloc(datum_len + 1);      \
+           T = (char*) _rb_safe_malloc(datum_len + 1); \
            strcpy(T, datum);                       \
        }                                           \
        continue;                                   \
@@ -123,6 +124,20 @@ extern double strtod();
        continue;                                  \
    }
 
+/* This macro added 2011 08 28 to allow use of 
+   of the 'setting' functions, such as 
+   p3_set_gs_primer_self_end, in setting
+   arguments. At this point, this macro is
+   only used twice, and there are no analogous
+   macros for integers,etc.
+*/
+#define COMPARE_FLOAT_USE_FN(TAG, set_function)        \
+   if (COMPARE(TAG)) {                                 \
+     parse_double(TAG, datum, &tmp_double, parse_err); \
+     set_function(pa, tmp_double);                     \
+     continue;                                         \
+   }
+
 #define COMPARE_INT(TAG,T)                     \
    if (COMPARE(TAG)) {                         \
        parse_int(TAG, datum, &(T), parse_err); \
@@ -168,24 +183,31 @@ read_boulder_record(FILE *file_input,
   pr_append_str *non_fatal_err;
   char *repeat_file_path = NULL, *int_repeat_file_path = NULL;
   int tmp_int;
-  int min_3_prime = 0, min_5_prime = 0;
+  /* int min_3_prime = 0, min_5_prime = 0; Removed 10/20/2010 */
   int min_3_prime_distance_global = 0;    /* needed to check if both global and specific*/
   int min_3_prime_distance_specific = 0;  /* are given in same boulder record */
   int min_three_prime_distance;           /* holder for the value of this tag */
+  double tmp_double;
 
   non_fatal_err = nonfatal_parse_err;
 
   while (((s = p3_read_line(file_input)) != NULL) && (strcmp(s,"="))) {
-    /* Read only the PRIMER tags if settings is selected */
-    /* Hint: strncomp returns 0 if both strings are equal */
-     if (file_type == settings && strncmp(s, "PRIMER_", 7)
-        && strncmp(s, "P3_FILE_ID", 10)) {
+    /* If we are reading from a settings file, then skip every 
+       line except those begining "PRIMER_" or "P3_FILE_ID".
+       Hint: strncomp returns 0 if both strings are equal */
+     if (file_type == settings 
+	 && strncmp(s, "PRIMER_", 7) /* "s does not begin with 'PRIMER_'" */
+	 && strncmp(s, "P3_FILE_ID", 10) /* "s does not begin with
+					    'P3_FILE_ID'" */
+	 ) {
       continue;
     }
     /* Silently ignore all primer3plus tags */
-    if (!(strncmp(s, "P3P_", 4))) {
+    /* Removed 10/20/10 because P3P_ tags
+       are already ignored in settings files. */
+    /* if (!(strncmp(s, "P3P_", 4))) {
       continue;
-    }
+      } */
 
     data_found = 1;
     /* Print out the input */
@@ -198,6 +220,7 @@ read_boulder_record(FILE *file_input,
        * of the record. */
       pr_append_new_chunk(glob_err, "Input line with no '=': ");
       pr_append(glob_err, s);
+      continue;
     } 
     /* Read in the old tags used until primer3 version 2.0 */
     else if (*io_version == 3) {
@@ -291,7 +314,7 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_PRODUCT_OPT_SIZE", pa->product_opt_size);
 
       /* =============================================================== */
-      /* Special handlng to presever backward compatible behavior
+      /* Special handlng to preserve backward compatible behavior
          for io version 3. */
       /* Old code was...
          COMPARE_ALIGN_SCORE("PRIMER_SELF_ANY", pa->p_args.max_self_any); */
@@ -371,8 +394,6 @@ read_boulder_record(FILE *file_input,
 		    pa->p_args.max_template_mispriming);
       COMPARE_FLOAT("PRIMER_PAIR_MAX_TEMPLATE_MISPRIMING",
                     pa->pair_max_template_mispriming);
-      COMPARE_FLOAT("PRIMER_INTERNAL_OLIGO_MAX_TEMPLATE_MISHYB",
-                    pa->o_args.max_template_mispriming);
 
       /* Control interpretation of ambiguity codes in mispriming
          and mishyb libraries. */
@@ -439,8 +460,6 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_IO_WT_REP_SIM", pa->o_args.weights.repeat_sim);
       COMPARE_FLOAT("PRIMER_IO_WT_SEQ_QUAL", pa->o_args.weights.seq_quality);
       COMPARE_FLOAT("PRIMER_IO_WT_END_QUAL", pa->o_args.weights.end_quality);
-      COMPARE_FLOAT("PRIMER_IO_WT_TEMPLATE_MISHYB",
-                    pa->o_args.weights.template_mispriming);
       COMPARE_FLOAT("PRIMER_PAIR_WT_PR_PENALTY", 
                     pa->pr_pair_weights.primer_quality);
       COMPARE_FLOAT("PRIMER_PAIR_WT_IO_PENALTY",
@@ -561,7 +580,15 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_MAX_NS_ACCEPTED", pa->p_args.num_ns_accepted);
       COMPARE_INT("PRIMER_PRODUCT_OPT_SIZE", pa->product_opt_size);
       COMPARE_FLOAT("PRIMER_MAX_SELF_ANY", pa->p_args.max_self_any);
-      COMPARE_FLOAT("PRIMER_MAX_SELF_END", pa->p_args.max_self_end);
+
+      /* COMPARE_FLOAT("PRIMER_MAX_SELF_END", pa->p_args.max_self_end); */
+      /* NEW */ COMPARE_FLOAT_USE_FN("PRIMER_MAX_SELF_END", p3_set_gs_primer_self_end)
+	/* if (COMPARE("PRIMER_MAX_SELF_END")) {
+	  parse_double("PRIMER_MAX_SELF_END", datum, &tmp_double, parse_err);
+	  p3_set_gs_primer_self_end(pa, tmp_double);
+	  continue;
+	  } */
+
       COMPARE_FLOAT("PRIMER_MAX_SELF_ANY_TH", pa->p_args.max_self_any_th);
       COMPARE_FLOAT("PRIMER_MAX_SELF_END_TH", pa->p_args.max_self_end_th);
       COMPARE_FLOAT("PRIMER_MAX_HAIRPIN_TH", pa->p_args.max_hairpin_th);   
@@ -569,7 +596,6 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_PAIR_MAX_COMPL_END", pa->pair_compl_end);
       COMPARE_FLOAT("PRIMER_PAIR_MAX_COMPL_ANY_TH", pa->pair_compl_any_th);
       COMPARE_FLOAT("PRIMER_PAIR_MAX_COMPL_END_TH", pa->pair_compl_end_th);
-      COMPARE_FLOAT("PRIMER_PAIR_MAX_HAIRPIN_TH", pa->pair_hairpin_th);
       COMPARE_INT("P3_FILE_FLAG", res->file_flag);
       COMPARE_INT("PRIMER_PICK_ANYWAY", pa->pick_anyway);
       COMPARE_INT("PRIMER_GC_CLAMP", pa->gc_clamp);
@@ -630,12 +656,12 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_SEQUENCING_ACCURACY", pa->sequencing.accuracy);
       if (COMPARE("PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION")) {
 	parse_int("PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION", datum, &pa->min_5_prime_overlap_of_junction, parse_err);
-	min_5_prime = 1;
+	/* min_5_prime = 1; Removed 10/20/2010 */
 	continue;
       }
       if (COMPARE("PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION")) {
 	parse_int("PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION", datum, &pa->min_3_prime_overlap_of_junction, parse_err);
-	min_3_prime = 1;
+	/* min_3_prime = 1; Removed 10/20/2010 */
 	continue;
       }
       COMPARE_AND_MALLOC("PRIMER_TASK", task_tmp);
@@ -664,8 +690,17 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_INTERNAL_MIN_QUALITY", pa->o_args.min_quality);
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_ANY",
 		    pa->o_args.max_self_any);
-      COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_END", 
-		    pa->o_args.max_self_end);
+
+      /* COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_END", 
+	 pa->o_args.max_self_end); */
+      /* NEW  */ COMPARE_FLOAT_USE_FN("PRIMER_INTERNAL_MAX_SELF_END", 
+				      p3_set_gs_primer_internal_oligo_self_end);
+      /* if (COMPARE("PRIMER_INTERNAL_MAX_SELF_END")) { 
+	 parse_double("PRIMER_INTERNAL_MAX_SELF_END", datum, &tmp_double, parse_err);
+	 p3_set_gs_primer_internal_oligo_self_end(pa, tmp_double);
+	 continue;
+	 } */
+
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_ANY_TH",
 			   pa->o_args.max_self_any_th);
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_END_TH",
@@ -687,12 +722,8 @@ read_boulder_record(FILE *file_input,
                     pa->pair_max_template_mispriming);
       COMPARE_FLOAT("PRIMER_PAIR_MAX_TEMPLATE_MISPRIMING_TH",
 		    pa->pair_max_template_mispriming_th);
-      COMPARE_FLOAT("PRIMER_INTERNAL_MAX_TEMPLATE_MISHYB",
-                    pa->o_args.max_template_mispriming);
-      COMPARE_FLOAT("PRIMER_INTERNAL_MAX_TEMPLATE_MISHYB_TH",
-			   pa->o_args.max_template_mispriming_th);
        /* Control interpretation of ambiguity codes in mispriming
-         and mishyb libraries. */
+          and mishyb libraries. */
       COMPARE_INT("PRIMER_LIB_AMBIGUITY_CODES_CONSENSUS",
                   pa->lib_ambiguity_codes_consensus);
       COMPARE_FLOAT("PRIMER_INSIDE_PENALTY", pa->inside_penalty);
@@ -782,8 +813,8 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_INTERNAL_WT_LIBRARY_MISHYB", pa->o_args.weights.repeat_sim);
       COMPARE_FLOAT("PRIMER_INTERNAL_WT_SEQ_QUAL", pa->o_args.weights.seq_quality);
       COMPARE_FLOAT("PRIMER_INTERNAL_WT_END_QUAL", pa->o_args.weights.end_quality);
-      COMPARE_FLOAT("PRIMER_INTERNAL_WT_TEMPLATE_MISHYB",
-                    pa->o_args.weights.template_mispriming);
+      COMPARE_FLOAT("PRIMER_WT_TEMPLATE_MISPRIMING_TH",
+		    pa->o_args.weights.template_mispriming_th);
       COMPARE_FLOAT("PRIMER_PAIR_WT_PR_PENALTY", 
                     pa->pr_pair_weights.primer_quality);
       COMPARE_FLOAT("PRIMER_PAIR_WT_IO_PENALTY",
@@ -835,34 +866,37 @@ read_boulder_record(FILE *file_input,
   /* Figure out the right settings for the tasks*/
   if (task_tmp != NULL) {
     if (!strcmp_nocase(task_tmp, "pick_pcr_primers")) {
-      pa->primer_task = pick_detection_primers;
+      pa->primer_task = generic;
       pa->pick_left_primer = 1;
       pa->pick_right_primer = 1;
       pa->pick_internal_oligo = 0;
     } else if (!strcmp_nocase(task_tmp, "pick_pcr_primers_and_hyb_probe")) {
-      pa->primer_task = pick_detection_primers; 
+      pa->primer_task = generic; 
       pa->pick_left_primer = 1;
       pa->pick_right_primer = 1;
       pa->pick_internal_oligo = 1;
     } else if (!strcmp_nocase(task_tmp, "pick_left_only")) {
-      pa->primer_task = pick_detection_primers;
+      pa->primer_task = generic;
       pa->pick_left_primer = 1;
       pa->pick_right_primer = 0;
       pa->pick_internal_oligo = 0;
     } else if (!strcmp_nocase(task_tmp, "pick_right_only")) {
-      pa->primer_task = pick_detection_primers;
+      pa->primer_task = generic;
       pa->pick_left_primer = 0;
       pa->pick_right_primer = 1;
       pa->pick_internal_oligo = 0;
     } else if (!strcmp_nocase(task_tmp, "pick_hyb_probe_only")) {
-      pa->primer_task = pick_detection_primers;
+      pa->primer_task = generic;
       pa->pick_left_primer = 0;
       pa->pick_right_primer = 0;
       pa->pick_internal_oligo = 1;
     } else if (*io_version == 3) {
       pr_append_new_chunk(glob_err, "Unrecognized PRIMER_TASK");
+    } else if (!strcmp_nocase(task_tmp, "generic")) {
+      pa->primer_task = generic;
     } else if (!strcmp_nocase(task_tmp, "pick_detection_primers")) {
-      pa->primer_task = pick_detection_primers;
+      pa->primer_task = generic; /* Deliberate duplication for
+				    backward compatibility. */
     } else if (!strcmp_nocase(task_tmp, "pick_cloning_primers")) {
       pa->primer_task = pick_cloning_primers;
     } else if (!strcmp_nocase(task_tmp, "pick_discriminative_primers")) {
@@ -892,11 +926,17 @@ read_boulder_record(FILE *file_input,
     free(task_tmp);
   }
 
-  /* WARNING: read_and_create_seq_lib uses p3_read_line, so repeat files cannot be read
-   * inside the while ((s = p3_read_line(stdin))...) loop above.
-   * FIX ME, in fact the reading of the library contents probably
-   * belongs inside primer3_boulder_main.c or libprimer3.c. */
-  /* Reading in the repeat libraries */
+  /* 
+   * WARNING: read_and_create_seq_lib uses p3_read_line, so repeat
+   * library files cannot be read inside the 
+   * while ((s = p3_read_line(stdin))...) loop above.
+   *
+   * FIX ME, in fact the reading
+   * of the library contents probably belongs inside
+   * primer3_boulder_main.c or libprimer3.c.
+   */
+
+  /* Read in the repeat libraries */
   if (NULL != repeat_file_path) {
     destroy_seq_lib(pa->p_args.repeat_lib);
     if ('\0' == *repeat_file_path) {
@@ -915,7 +955,7 @@ read_boulder_record(FILE *file_input,
     repeat_file_path = NULL;
   }
 
-  /* Reading in the repeat libraries for internal oligo */
+  /* Read in the repeat libraries for internal oligo */
   if (NULL != int_repeat_file_path) {
     destroy_seq_lib(pa->o_args.repeat_lib);
     if ('\0' == *int_repeat_file_path) {
@@ -936,9 +976,10 @@ read_boulder_record(FILE *file_input,
 
   /* Fix very old tags for backward compatibility */
   if (*io_version == 3) {
+
     /* This next belongs here rather than libprimer3, because it deals
-       with potential incompatibility with old tags (kept for backward
-       compatibility, and new tags.  */
+       with potential incompatibility between new tags and old tags
+       (which we have kept for backward compatibility).  */
     if (pa->primer_task == pick_pcr_primers_and_hyb_probe) {
       PR_ASSERT(pa->pick_internal_oligo);
     }
@@ -977,12 +1018,14 @@ read_boulder_record(FILE *file_input,
     PR_ASSERT(pa->pick_internal_oligo);
   }
 
-  if ((min_3_prime || min_5_prime) && (sa->primer_overlap_junctions_count == 0)) {
-    pr_append_new_chunk(warnings,
-			"SEQUENCE_OVERLAP_JUNCTION_LIST not given, but "
-			"PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION or "
-			"PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION specified");
-  }
+  /* Removed 10/20/2010 as excessively compulsive, especially in the context
+     of taking input from web pages.
+     if ((min_3_prime || min_5_prime) && (sa->primer_overlap_junctions_count == 0)) {
+     pr_append_new_chunk(warnings,
+                         "SEQUENCE_OVERLAP_JUNCTION_LIST not given, but "
+			 "PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION or "
+			 "PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION specified");
+			 } */
 
   return 1;
 }
@@ -992,10 +1035,19 @@ read_boulder_record(FILE *file_input,
 #undef COMPARE_FLOAT
 #undef COMPARE_INTERVAL_LIST
 
+static void
+pr_append2(pr_append_str *err,
+	   const char* s1, 
+	   const char* s2) {
+    pr_append_new_chunk(err, s1);
+    pr_append(err, s2);
+}  
+
 int 
 read_p3_file(const char *file_name,
 	     const p3_file_type expected_file_type,
 	     int echo_output,
+	     int strict_tags,
 	     p3_global_settings *pa, 
 	     seq_args *sa,
 	     pr_append_str *fatal_err,
@@ -1003,64 +1055,94 @@ read_p3_file(const char *file_name,
 	     pr_append_str *warnings,
 	     read_boulder_record_results *read_boulder_record_res) 
 {
-  /* Parameter for read_boulder_record */
   FILE *file;
   int ret_par = 0;
-  int strict_tags = 0;
   int io_version = 4;
-  char first_line[80];
-  char second_line[80];
-  int error = 0;
+  char *line1;
+  char *line2;
+  char *line3;
   p3_file_type file_type = all_parameters;
     
   /* Check if a file name was provided */
   PR_ASSERT(NULL != file_name);
+
   /* Open the file */
-  if((file = fopen(file_name,"r")) != NULL) {
-        /* Deal with the file headers */
-    strncpy (first_line, p3_read_line(file), 79);
-    strncpy (second_line, p3_read_line(file), 79);
-    p3_read_line(file);
-
-    if (strncmp(first_line,"Primer3 File - http://primer3.sourceforge.net", 45)) {
-        error = 1;
-    }
-    if (!strncmp(second_line,"P3_FILE_TYPE=all_parameters", 27)) {
-      file_type = all_parameters;
-    }
-    else if (!strncmp(second_line,"P3_FILE_TYPE=sequence", 21)) {
-      file_type = sequence;
-    }
-    else if (!strncmp(second_line,"P3_FILE_TYPE=settings", 21)) {
-      file_type = settings;
-    }
-    else {
-      error = 1;
-    }
-
-    if (echo_output) {
-      printf("P3_SETTINGS_FILE_USED=%s\n", file_name);
-      printf("%s\n", second_line);
-    }
-
-    /* Check if the file type matches the expected type */
-    if (file_type != expected_file_type){
-      pr_append_new_chunk(nonfatal_err, 
-                          "Unexpected P3 file type parsed");
-    }
-    /* read FILE_TYPE */
-    if (error == 0){
-      ret_par = read_boulder_record(file, &strict_tags, &io_version, 
-				    echo_output, expected_file_type, pa, sa, fatal_err, 
-				    nonfatal_err, warnings, read_boulder_record_res);
-    } else {
-      pr_append_new_chunk(fatal_err, "Incorrect file format in ");
-      pr_append(fatal_err, file_name);
-    }
-  } else {
-    pr_append_new_chunk(fatal_err, "Cannot open ");
-    pr_append(fatal_err, file_name);
+  if (!(file = fopen(file_name,"r"))) {
+    pr_append2(fatal_err, "Cannot open ", file_name);
+    return ret_par;
   }
+
+  /* Parse and interpret the first 3 lines */
+
+  /* Line 1 */
+  line1 = p3_read_line(file);
+  if (!line1) {
+    pr_append2(fatal_err, 
+	       "Settings file is empty: ", 
+	       file_name);
+    return ret_par;
+  }
+  if (strcmp(line1,
+	     "Primer3 File - http://primer3.sourceforge.net")) {
+      pr_append2(fatal_err,
+		 "First line must be \"Primer3 File - http://primer3.sourceforge.net\" in ",
+		 file_name);
+      return ret_par;
+  }
+
+  /* Line 2 */
+  line2 = p3_read_line(file);
+  if (!line2) {
+    pr_append2(fatal_err, 
+	       "Incorrect file format (too few lines) in ", 
+	       file_name);
+    return ret_par;
+  }
+  if (!strcmp(line2,"P3_FILE_TYPE=all_parameters")) {
+    file_type = all_parameters;
+  } else if (!strcmp(line2,"P3_FILE_TYPE=sequence")) {
+    file_type = sequence;
+  } else if (!strcmp(line2,"P3_FILE_TYPE=settings")) {
+    file_type = settings;
+  } else {
+    pr_append2(fatal_err, 
+	       "Unknown file type in at line 2 (line2='", line2);
+    pr_append(fatal_err, "') in ");
+    pr_append(fatal_err, file_name);
+    return ret_par;
+  }
+  if (echo_output) {
+    printf("P3_SETTINGS_FILE_USED=%s\n", file_name);
+    printf("%s\n", line2);
+  }
+
+  /* Line 3 */
+  line3 = p3_read_line(file);
+  if (!line3) {
+    pr_append2(fatal_err, 
+	       "Incorrect file format (too few lines) in ", 
+	       file_name);
+      return ret_par;
+  }
+  if (strcmp(line3, "")) {
+    pr_append2(fatal_err, "Line 3 must be empty in ",
+	       file_name);
+    return ret_par;
+  }
+
+  /* Check if the file type matches the expected type */
+  if (file_type != expected_file_type){
+    pr_append_new_chunk(nonfatal_err, 
+			"Unexpected P3 file type parsed");
+  }
+
+  /* read the file */
+  ret_par = read_boulder_record(file, &strict_tags, &io_version, 
+				echo_output, expected_file_type,
+				pa, sa, fatal_err, 
+				nonfatal_err, warnings, 
+				read_boulder_record_res);
+
   if (echo_output) printf("P3_SETTINGS_FILE_END=\n");
   if (file) fclose(file);
          
@@ -1206,10 +1288,10 @@ parse_interval_list(const char *tag_name,
 
 /*
  * For correct input, return a pointer to the first non-tab, non-space
- * character after the forth integer and after the separator sep2, and place the integers in out1,
- * out2, out3 and out4  On incorrect input, return NULL;
- * If any of the 4 integers is not specified, the corresponding output
- * value will be -1.
+ * character after the forth integer and after the separator sep2, and
+ * place the integers in out1, out2, out3 and out4. On incorrect input,
+ * return NULL; If any of the 4 integers is not specified, the
+ * corresponding output value will be -1.
  */
 static char *
 parse_2_int_pair(const char    *tag_name, char *datum,
