@@ -1,7 +1,8 @@
 /*
- Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2009,2010
+ Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2009,2010,
+               2011,2012
  Whitehead Institute for Biomedical Research, Steve Rozen
- (http://jura.wi.mit.edu/rozen), and Helen Skaletsky
+ (http://purl.com/STEVEROZEN/), and Helen Skaletsky
  All rights reserved.
 
        This file is part of primer3 software suite.
@@ -61,7 +62,7 @@
 #endif
 
 #ifndef THAL_MAX_SEQ
-#define THAL_MAX_SEQ   1600
+#define THAL_MAX_SEQ   10000
 #endif
 
 /* table where bp-s enthalpies, that retrieve to the most stable Tm, are saved */
@@ -86,7 +87,7 @@
 # define HEND5(i) hend5[i]
 #endif
 
-#define CHECK_ERROR(COND,MSG) if (COND) { strcpy(o->msg, MSG); longjmp(_jmp_buf, 1); }
+#define CHECK_ERROR(COND,MSG) if (COND) { strcpy(o->msg, MSG); errno = 0; longjmp(_jmp_buf, 1); }
 #define THAL_OOM_ERROR { strcpy(o->msg, "Out of memory"); errno = ENOMEM; longjmp(_jmp_buf, 1); }
 #define THAL_IO_ERROR(f) { sprintf(o->msg, "Unable to open file %s", f); longjmp(_jmp_buf, 1); }
 
@@ -317,8 +318,10 @@ static struct tetraloop* tetraloopEntropies = NULL; /* ther penalties for given
 static struct tetraloop* tetraloopEnthalpies = NULL; /* ther penalties for given tetraloop seq-s */
 static jmp_buf _jmp_buf;
 
-
-/* Function to read the thermodynamic values from the parameter files, returns 0 in case of success */
+/* Read the thermodynamic values (parameters) from the parameter files
+   in the directory specified by 'path'.  Return 0 on success and -1
+   on error. The thermodynamic values are stored in multiple static
+   variables. */
 int 
 get_thermodynamic_values(const char* path, thal_results *o)
 {
@@ -381,21 +384,28 @@ thal(const unsigned char *oligo_f,
    oligo1 = oligo2 = NULL;
    strcpy(o->msg, "");
    o->temp = THAL_ERROR_SCORE;
+   errno = 0; 
 
    if (setjmp(_jmp_buf) != 0) {
      o->temp = THAL_ERROR_SCORE;
      return;  /* If we get here, that means we returned via a
-                 longjmp.  In this case errno should be ENOMEM. */
+                 longjmp.  In this case errno might be ENOMEM,
+		 but not necessarily. */
    }
 
    CHECK_ERROR(NULL == oligo_f, "NULL first sequence");
    CHECK_ERROR(NULL == oligo_r, "NULL second sequence");
    len_f = length_unsig_char(oligo_f);
    len_r = length_unsig_char(oligo_r);
+
+   /* The following error messages will be seen by end users and will
+      not be easy to understand. */
    CHECK_ERROR((len_f > THAL_MAX_ALIGN) && (len_r > THAL_MAX_ALIGN),
 	       "Sequences longer than THAL_MAX_ALIGN for thermodynamical alignment (nearest-neighbor approach)");
-   CHECK_ERROR((len_f > THAL_MAX_SEQ), "Sequence 1 longer than THAL_MAX_SEQ and alignment is requested");
-   CHECK_ERROR((len_r > THAL_MAX_SEQ), "Sequence 2 longer than THAL_MAX_SEQ and alignment is requested");
+   CHECK_ERROR((len_f > THAL_MAX_SEQ), 
+	       "Sequence 1 longer than THAL_MAX_SEQ and alignment is requested");
+   CHECK_ERROR((len_r > THAL_MAX_SEQ), 
+	       "Sequence 2 longer than THAL_MAX_SEQ and alignment is requested");
 
    CHECK_ERROR(NULL == a,  "NULL 'in' pointer");
    if (NULL == o) return; /* Leave it to the caller to crash */
@@ -465,6 +475,7 @@ thal(const unsigned char *oligo_f,
    } else {
       strcpy(o->msg, "Wrong alignment type!");
       o->temp = THAL_ERROR_SCORE;
+      errno=0;
 #ifdef DEBUG
       fprintf(stderr, o->msg);
 #endif
@@ -733,8 +744,8 @@ openParamFile(const char* fname, thal_results* o)
 #ifdef DEBUG
 	 perror(paramdir);
 #endif
+	 THAL_IO_ERROR(paramdir);
 	 free(paramdir);
-	 THAL_IO_ERROR(fname);
       }
       free(paramdir);
    }
