@@ -4,18 +4,18 @@
 # For usage, see the usage statement in the code, below.
 #
 # ======================================================================
-# (c) Copyright 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008
+# (c) Copyright 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008,2010,
+#  2011,2012
 # Whitehead Institute for Biomedical Research, Steve Rozen, 
 # Andreas Untergasser and Helen Skaletsky
 # All rights reserved.
 # 
 #   This file is part of the primer3 suite.
 #
-#   The primer3 suite is free software; you can
-#   redistribute them and/or modify them under the terms of the GNU
-#   General Public License as published by the Free Software Foundation;
-#   either version 2 of the License, or (at your option) any later
-#   version.
+#   The primer3 suite is free software; you can redistribute it and/or
+#   modify it under the terms of the GNU General Public License as
+#   published by the Free Software Foundation; either version 2 of the
+#   License, or (at your option) any later version.
 #
 #   This software is distributed in the hope that it will be useful,
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -57,7 +57,7 @@ sub _nowarn_system($);
 our $def_executable = "../src/primer3_core";
 our $exe = '../src/primer3_core';
 our $set_files = '../test/';
-our ($verbose, $do_valgrind, $winFlag, $fastFlag);
+our ($verbose, $do_valgrind, $winFlag, $fastFlag, $onetest);
 
 our %signo;
 
@@ -71,7 +71,8 @@ main();
 sub main() {
     my %args;
 
-    select STDERR; $| = 1;
+    # select STDERR;
+    $| = 1;
 
     $all_ok = 1;
     my $start_time;
@@ -93,10 +94,12 @@ sub main() {
                     'windows',
                     'fast',
                     'verbose',
+		    'onetest=s',
                     'executable=s',
                     )) {
         print "Usage: perl p3test.pl \\\n",
-        "    [--executable <primer3 executable>] [ --valgrind ] [  --verbose ] [--windows] [--fast]\n",
+        "    [--executable <primer3 executable>] [ --onetest <test_name> ] ",
+	"[ --valgrind ] [  --verbose ] [--windows] [--fast]\n",
         "\n",
         "    where <primer3 executable> defaults to ../src/primer3_core\n";
         exit -1;
@@ -116,7 +119,7 @@ sub main() {
     my $log_file_arg_for_valgrind = "--log-file-exactly";
     if ($do_valgrind) {
 	if (!-x $valgrind_exe) { 
-	    warn "Cannot find $valgrind_exe; will try `which valgrind`\n";
+	    print "Cannot find $valgrind_exe; will try `which valgrind`\n";
 	    $valgrind_exe= `which valgrind`;
 	    chomp($valgrind_exe);
 	    if (!$valgrind_exe || ! -x $valgrind_exe) {
@@ -152,12 +155,104 @@ sub main() {
     print "verbose mode\n" if $verbose;
     print "valgrind mode\n" if $do_valgrind;
 
-    test_fatal_errors();
+    # Tests in %default_version2 use --default_version=2.
+    # Others use -default_version=1.
+    my %default_version2 = ('th-w-other-tasks' => 1,
+			    'primer_thermod_align' => 1,
+			    'primer_thermod_align_formatted' => 1,
+			    'primer1_th' => 1,
+			    'primer_mispriming_th' => 1,
+			    'primer_must_use_th' => 1,
+			    'primer_new_tasks_th' => 1,
+			    'primer_task_th' => 1,
 
-    # The range of this for loop is a set of test names
-    # that get translated into file names inside the loop.
-    for my $test (
-	
+			    # 'test_compl_error' => 1,
+			    # 'test_left_to_right_of_right' => 1,
+
+			    'primer_boundary' => 1, # Put the quickest tests first.
+
+			    # 'primer_boundary1' => 1,
+			    # 'primer_boundary_formatted' => 1,
+			    # 'primer_boundary1_formatted' => 1,
+			    
+			    # 'primer_internal' => 1,
+			    # 'primer_internal1' => 1,
+			    # 'primer_internal_formatted' => 1,
+			    # 'primer_internal1_formatted' => 1,
+			    
+			    # 'primer_ok_regions' => 1,
+			    # 'primer_ok_regions_formatted' => 1,
+			    # 'primer_ok_regions2' => 1,
+
+			    # 'primer_tm_lc_masking' => 1,
+			    # 'primer_tm_lc_masking_formatted' => 1,
+
+			    # 'primer_start_codon' => 1,
+			    
+			    # 'primer_task' => 1,
+			    # 'primer_task_formatted' => 1,
+			    # 'primer_renewed_tasks' => 1,
+			    # 'primer_new_tasks' => 1,
+			    # 'primer_new_tasks_formatted' => 1,
+			    
+			    # 'primer_must_overlap_point' => 1,
+			    # 'primer_overlap_junction' => 1,
+			    
+			    # 'primer_all_settingsfiles' => 1,
+			    # 'primer_high_tm_load_set' => 1,
+			    # 'primer_high_gc_load_set' => 1,
+
+			    # 'primer_gc_end' => 1,
+			    # 'primer_check' => 1,
+			    # 'primer_must_use' => 1,
+			    # 'primer_must_use_formatted' => 1,
+			    # 'primer_syntax' => 1,
+			    # 'primer_end_pathology' => 1,
+			    # 'primer_num_best' => 1,
+			    # 'primer_quality_boundary' => 1,
+			    # 'primer' => 1,
+			    # 'primer1' => 1,
+			    # 'primer_mispriming' => 1,
+			    # 'primer_mispriming_formatted' => 1,
+			    # 'primer_mispriming_boundary1' => 1,
+			    # 'primer_mispriming_boundary1_formatted' => 1,
+			    # 'primer_mispriming_boundary2' => 1,
+			    # 'primer_mispriming_boundary2_formatted' => 1,
+			    # 'primer_mispriming_long_lib' => 1,
+			    # 'primer_rat' => 1,
+			    # 'primer_human' => 1,
+			    # 'long_seq' => 1,
+			    # 'primer_position_penalty' => 1,
+			    # 'primer_position_penalty_formatted' => 1,
+			    # 'p3-tmpl-mispriming' => 1,
+
+			    # Put slow tests last
+
+			    # 'p3_3_prime_n' => 1,
+			    # 'primer_three_prime_distance' => 1,
+			    
+			    # 'primer_obj_fn' => 1,
+			    
+			    # 'primer_lib_amb_codes' => 1,
+			    );
+
+    my @TESTS = ( 
+		  'th-w-other-tasks',
+
+	          'primer_thermod_align',             
+	          'primer_thermod_align_formatted',
+
+		  # New tests that use new melting temperature
+		  # and thermodynamic alignments
+		  'primer1_th',
+		  'primer_mispriming_th',
+		  'primer_must_use_th',
+		  'primer_new_tasks_th',
+		  'primer_task_th',
+
+
+		  'test_compl_error',
+		  'test_left_to_right_of_right',
 	          'primer_boundary', # Put the quickest tests first.
                   'primer_boundary1',
                   'primer_boundary_formatted',
@@ -217,26 +312,46 @@ sub main() {
 
                   'p3_3_prime_n',
 	          'primer_three_prime_distance',
-	
-	          'primer_thermod_align',             
-	          'primer_thermod_align_formatted',
-	
+
                   'primer_obj_fn',
 
                   'primer_lib_amb_codes',
-                  ) {
+
+		  );
+
+
+    if (!$args{'onetest'}) {
+	test_fatal_errors();
+    } else {
+	@TESTS = $args{'onetest'};
+    }
+
+    # The range of this for loop is a set of test names
+    # that get translated into file names inside the loop.
+    for my $test (@TESTS) {
 
         # We are inside the for loop here....
         print "$test...";
 
         if ($fastFlag && (($test eq 'p3_3_prime_n')
-                || ($test eq 'primer_obj_fn')
+            || ($test eq 'th-w-other-tasks')
+            || ($test eq 'primer_obj_fn')
+            || ($test eq 'primer1_th')
+            || ($test eq 'primer_mispriming_th')
+            || ($test eq 'primer_new_tasks_th')
 	        || ($test eq 'primer_thermod_align')
 	        || ($test eq 'primer_thermod_align_formatted'))) {
             print "[skiped in fast mode]\n";
             next;
         }
 
+	my $default_version;
+	if ($default_version2{$test}) {
+	    $default_version = '-default_version=2';
+	} else {
+	    $default_version = '-default_version=1';
+	}
+
         if ($test eq 'primer_lib_amb_codes') {
             if ($fastFlag) {
                 print "[skiped in fast mode]\n";       
@@ -263,7 +378,13 @@ sub main() {
 
         my $r;                  # Return value for tests
 
-        if ($test eq 'primer' || $test eq 'primer1') {
+	my %list_test 
+	    = ('primer' => 1,
+	       'primer1' => 1,
+	       'primer1_th' => 1,
+	       'th-w-other-tasks' => 1);
+
+        if (exists($list_test{$test})) {
             # These tests generate primer lists, which
             # need to be checked separately.
 
@@ -284,22 +405,33 @@ sub main() {
             # generate the necc. files; If $winFlag is 
             # set, run command with Windows backslashes
             # in path.
+	    my $settings = '';
+	
             if ($winFlag) {
-                $tmpCmd = "..\\$exe -strict_tags <../$input >../$tmp";
+		if ($test eq 'th-w-other-tasks') {
+		    $settings = '-p3_settings ..\\th-w-other-tasks-settings.txt -echo_settings';
+		}
+		$tmpCmd = "..\\$exe $default_version -strict_tags $settings ../$input >../$tmp";
             }  else {
-                $tmpCmd = "$valgrind_prefix ../$exe -strict_tags <../$input >../$tmp";
+		if ($test eq 'th-w-other-tasks') {
+		    $settings = '-p3_settings ../th-w-other-tasks-settings.txt -echo_settings';
+		}
+                $tmpCmd = "$valgrind_prefix ../$exe $default_version -strict_tags $settings ../$input >../$tmp";
             }
             $r = _nowarn_system($tmpCmd);
             # back to main directory
             if (!chdir "../") { die "chdir \"..\": $!\n" }
+
         } elsif ($test =~ /formatted$/) {
-            my $cmd = "$valgrind_prefix$exe -strict_tags -format_output <$input >$tmp";
+            my $cmd = "$valgrind_prefix$exe $default_version -strict_tags -format_output <$input >$tmp";
             $r = _nowarn_system($cmd);
+
         } elsif ($test =~ /_load_set/) {
-            my $cmd = "$valgrind_prefix$exe -strict_tags -p3_settings_file=$set_files$test.set -echo <$input >$tmp";
+            my $cmd = "$valgrind_prefix$exe $default_version -strict_tags -p3_settings_file=$set_files$test.set -echo <$input >$tmp";
             $r = _nowarn_system($cmd);
+
         } else {
-            my $cmd = "$valgrind_prefix$exe -strict_tags <$input >$tmp";
+            my $cmd = "$valgrind_prefix$exe $default_version -strict_tags <$input >$tmp";
             $r = _nowarn_system($cmd);
         }
 
@@ -319,9 +451,15 @@ sub main() {
             $exit_stat = -1;
         }
 
-        if ($test eq 'primer' || $test eq 'primer1') {
+        if (exists($list_test{$test})) {
+	    # Special processing for tests that generate files containing
+	    # primer lists.
             my $list_tmp = $test.'_list_tmp';
             my $list_last = $test.'_list_last';
+	    if (!-r $list_last) {
+		print "Configuration error, $list_last does not exist or is not readable ";
+		$r = 1;
+	    }
             my @tempList = glob($list_last.'/*');
             # do _file by file_ comparison within primer_list_last to primer_list_tmp
             # since we are not using diff - diff used to do directory comparison
@@ -357,23 +495,27 @@ sub main() {
         my $r = system "grep ERROR *.vg */*.vg | grep -v 'ERROR SUMMARY: 0 errors'";
         if (!$r) { # !$r because grep returns 0 if something is found,
             # and if something is found, we have a problem.
+	    $all_ok = 0;
+	    print "\nValgrind ERRORs found [FAILED]";
             $exit_stat = -1;
         }
         $r = system "grep 'definitely lost' *.vg */*.vg | grep -v ' 0 bytes'";
         if (!$r) {
+	    print "\nValgrind LEAKSs found [WARNING]";
             $exit_stat = -1;
         }
         $r = system "grep 'possibly lost' *.vg */*.vg   | grep -v ' 0 bytes'";
         if (!$r) {
+	    print "\nValgrind LEAKSs found [WARNING]";
             $exit_stat = -1;
         }
     }
-    print "Tests ran for ", (time() - $start_time), " seconds.\n";
+    print "\nTests ran for ", (time() - $start_time), " seconds.\n";
     print "\n\nDONE ", scalar(localtime), " ";
 
     print $all_ok ? "Passed all tests - [OK]\n\n\n" : "At least one test failed - [FAILED]\n\n\n";
 
-    exit $exit_stat;
+    exit 0 # $exit_stat; Change here, generally we want the testing to continue.
 }
 
 # Usage: perldiff("filename1", "filename2")
@@ -464,8 +606,9 @@ sub test_fatal_errors() {
     # Get all the files that match primer_global_err/*.in:
     my @inputs = glob("./primer_global_err/*.in");
     my $r;
-    my $problem = 0;
+    my $fatal_error_problem = 0;
     print "\ntesting fatal errors...\n";
+    my $output_and_err_tested = 0;
     for (@inputs) {
         my ($root) = /(.*)\.in$/;  # Hint, the parens around $root give
                                    # the result of the match in
@@ -474,7 +617,23 @@ sub test_fatal_errors() {
         my $valgrind_prefix
             = $do_valgrind ? sprintf $valgrind_format, $root : '';
 
-        my $cmd = "$valgrind_prefix$exe <$_ > $root.tmp 2> $root.tmp2";
+        my $cmd;
+	if ($_ =~ /bad_settings\d\.in/) {
+	    # For testing the settings files we need the
+	    # names of tests and the settings to be parallel
+	    $cmd = "$valgrind_prefix$exe -strict_tags -p3_settings_file $_ primer_global_err/input_for_settings_tests.txt  > $root.tmp 2> $root.tmp2";
+	    # print STDERR $cmd, "\n";
+	} else {
+	    if ($output_and_err_tested) {
+		$cmd = "$valgrind_prefix$exe -strict_tags <$_ > $root.tmp 2> $root.tmp2";
+	    } else {
+		# We need to test the -output and -error command line arguments plus
+                # simply taking the file name as an argument (no "<")
+		$cmd = "$valgrind_prefix$exe -strict_tags $_ -output $root.tmp -error $root.tmp2";
+		$output_and_err_tested = 1;
+		print "Testing -output and -error flags on\n$cmd\n";
+	    }
+	}
 
 	$r = _nowarn_system($cmd);
 
@@ -482,23 +641,23 @@ sub test_fatal_errors() {
             my $r = $? >> 8;
             print
                 "\nErroneous 0 exit status ($?) from command $cmd\n";
-            $problem = 1;
+            $fatal_error_problem = 1;
         }
         if (perldiff "$root.tmp", "$root.out") {
             print
                 "Difference found between $root.out and $root.tmp\nfrom $cmd\n\n";
-            $problem = 1;
+            $fatal_error_problem = 1;
         }
         if (perldiff "$root.tmp2", "$root.out2") {
             print 
                 "\nDifference found between $root.out2 and $root.tmp2\nfrom $cmd\n\n";
-            $problem = 1;
+            $fatal_error_problem = 1;
         }
     }
-    if ($problem == 1){
+    if ($fatal_error_problem == 1){
         $all_ok = 0;
     }
-    print $problem ? "[FAILED]" : "[OK]" ,"\n";
+    print $fatal_error_problem ? "[FAILED]" : "[OK]" ,"\n";
 }
 
 sub _nowarn_system($) {
@@ -516,6 +675,9 @@ sub _nowarn_system($) {
                 print "Presumably caused by catching SIGINT\n";
                 print "Tests halted\n\n";
                 print "\nWARNING: not all tests executed ... [FAILED]\n";
+		if (!$all_ok) {
+		    print "\n\nIn addition, at least 1 test FAILED ... [FAILED]\n\n";
+		}
                 exit;
             }
         }
